/*
  Copyright 2009 Jason Wells 

  tictactoe - play a simple two-player game of Tic Tac Toe 

  This file is a part of Tic Tac Toe.

  Tic Tac Toe is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  any later version.

  Tic Tac Toe is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Tic Tac Toe.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <getopt.h>

#include "error.h"
#include "iface.h"

const char version[] = "Tic-tac-toe 1.0";
const char help[] = "Turns alternate between X and O. X moves first. Click a box to move.";

/* board represents a 3x3 tic-tac-toe board */
char board[9] = "         ";

/* mark: place the given character (X or O) at the position n */
void
mark (int n, char c)
{
  if (c == 'X' || c == 'O')
    if (n >= 0 && n < 9)
      if (board[n] == ' ')
	board[n] = c;
}

/* iswin: examine game board to see if there is a winner */
bool
iswin (void)
{
  /* check columns */
  for (int i = 0; i < 3; i++)
    if (board[i] == board[i + 3] && board[i + 3] == board[i + 6] && board[i] != ' ')
      return true;

  /* check rows */
  for (int i = 0; i < 9; i += 3)
    if (board[i] == board[i + 1] && board [i + 1] == board[i + 2] && board[i] != ' ')
      return true;

  /* check diagonals */
  if (board[0] == board[4] && board[4] == board[8] && board[0] != ' ')
    return true;

  if (board[2] == board[4] && board[4] == board[6] && board[2] != ' ')
    return true;
    
  return false;
}

int
main (int argc, char *argv[])
{
  if (argc > 1)
    {
      /* parse --version and --help options */
      int c;
      int ip;

      static struct option longopt[] =
	{
	  {"version", no_argument, NULL, 'v'},
	  {"help", no_argument, NULL, 'h'} 
	};

      while ((c = getopt_long (argc, argv, "vh", longopt, &ip)))
	{
	  switch (c)
	    {
	    case 'v':
	      puts (version);
	      exit (EXIT_SUCCESS);
	      break;

	    case 'h':
	      puts (help);
	      exit (EXIT_SUCCESS);
	      break;

	    default:
	      break;
	    }
	}
    }
  
  /* set up graphic interface */
  if (initiface () == false)
    {
      eprintf ("%s:%d: failed to initialize interface\n", __FILE__, __LINE__);
      exit (EXIT_FAILURE);
    }

  /* main game loop */
  for (bool done = false; !done; )
    {
      const char ch[2] = "XO";

      /* alternate turns between X and O */
      for (int i = 0; i < 2 && !done; i++)
	{
	  int j = move ();
	  mark (j, ch[i]);
	  show ();

	  if ((done = iswin ()))
	    winalert (ch[i]);
	}           
    }

  exit (EXIT_SUCCESS);
}
